#!/usr/bin/env python

import sys
import yaml
import yaml_odict
from collections import OrderedDict

from pprint import pprint

registers = yaml.load(open(sys.argv[1], 'r'))

for register_name, register in registers.iteritems():
    print('/* --- %s values %s */' % (register_name, '-' * (50 - len(register_name))))
    print
    fields = register['fields']
    #for field_name, field in sorted(fields.items(), lambda x, y: cmp(x[1]['lsb'], y[1]['lsb'])):
    for field_name, field in fields.items():
        mask_bits = (1 << field['width']) - 1
        print('/* %s: %s */' % (field_name, field['description']))
        print('#define %s_%s_SHIFT (%d)' % (
            register_name, field_name, field['lsb'],
        ))
        print('#define %s_%s_MASK (0x%x << %s_%s_SHIFT)' % (
            register_name, field_name, mask_bits, register_name, field_name,
        ))
        print('#define %s_%s(x) ((x) << %s_%s_SHIFT)' % (
            register_name, field_name, register_name, field_name,
        ))
        print



import yaml
from collections import OrderedDict
def construct_odict(load, node):
    """This is the same as SafeConstructor.construct_yaml_omap(),
    except the data type is changed to OrderedDict() and setitem is
    used instead of append in the loop.
 
    >>> yaml.load('''
    ... !!omap
    ... - foo: bar
    ... - mumble: quux
    ... - baz: gorp
    ... ''')
    OrderedDict([('foo', 'bar'), ('mumble', 'quux'), ('baz', 'gorp')])
 
    >>> yaml.load('''!!omap [ foo: bar, mumble: quux, baz : gorp ]''')
    OrderedDict([('foo', 'bar'), ('mumble', 'quux'), ('baz', 'gorp')])
    """
 
    omap = OrderedDict()
    yield omap
    if not isinstance(node, yaml.SequenceNode):
        raise yaml.constructor.ConstructorError(
            "while constructing an ordered map",
            node.start_mark,
            "expected a sequence, but found %s" % node.id, node.start_mark
        )
    for subnode in node.value:
        if not isinstance(subnode, yaml.MappingNode):
            raise yaml.constructor.ConstructorError(
                "while constructing an ordered map", node.start_mark,
                "expected a mapping of length 1, but found %s" % subnode.id,
                subnode.start_mark
            )
        if len(subnode.value) != 1:
            raise yaml.constructor.ConstructorError(
                "while constructing an ordered map", node.start_mark,
                "expected a single mapping item, but found %d items" % len(subnode.value),
                subnode.start_mark
            )
        key_node, value_node = subnode.value[0]
        key = load.construct_object(key_node)
        value = load.construct_object(value_node)
        omap[key] = value
 
yaml.add_constructor(u'tag:yaml.org,2002:omap', construct_odict)

def repr_pairs(dump, tag, sequence, flow_style=None):
    """This is the same code as BaseRepresenter.represent_sequence(),
    but the value passed to dump.represent_data() in the loop is a
    dictionary instead of a tuple."""
 
    value = []
    node = yaml.SequenceNode(tag, value, flow_style=flow_style)
    if dump.alias_key is not None:
        dump.represented_objects[dump.alias_key] = node
    best_style = True
    for (key, val) in sequence:
        item = dump.represent_data({key: val})
        if not (isinstance(item, yaml.ScalarNode) and not item.style):
            best_style = False
        value.append(item)
    if flow_style is None:
        if dump.default_flow_style is not None:
            node.flow_style = dump.default_flow_style
        else:
            node.flow_style = best_style
    return node

def repr_odict(dumper, data):
    """
    >>> data = OrderedDict([('foo', 'bar'), ('mumble', 'quux'), ('baz', 'gorp')])
    >>> yaml.dump(data, default_flow_style=False)
    '!!omap\\n- foo: bar\\n- mumble: quux\\n- baz: gorp\\n'
    >>> yaml.dump(data, default_flow_style=True)
    '!!omap [foo: bar, mumble: quux, baz: gorp]\\n'
    """
    return repr_pairs(dumper, u'tag:yaml.org,2002:omap', data.iteritems())
 
yaml.add_representer(OrderedDict, repr_odict)
 



#!/usr/bin/env python

import sys
import yaml
import csv
from collections import OrderedDict
import yaml_odict
 
def convert_file(fname):
    reader = csv.reader(open(fname, 'r'))

    registers = OrderedDict()
    for register_name, lsb, width, field_name, description, reset_value, access in reader:
        if register_name not in registers:
            registers[register_name] = {
                'fields': OrderedDict(),
            }

        register = registers[register_name]
        fields = register['fields']
        if field_name in fields:
            raise RuntimeError('Duplicate field name "%s" in register "%s"' %
                    field_name, register_name)
        else:
            fields[field_name] = {
                'lsb': int(lsb),
                'width': int(width),
                'description': description,
                'reset_value': reset_value,
                'access': access,
            }

    with open(fname.replace('.csv', '.yaml'), 'w') as out_file:
        yaml.dump(registers, out_file, default_flow_style=False)

for fname in sys.argv[1:]:
    convert_file(fname)
